package com.rjconsultores.ventaboletos.web.gui.controladores.tarifas;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Cotacao;
import com.rjconsultores.ventaboletos.entidad.Moneda;
import com.rjconsultores.ventaboletos.service.MonedaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCotacao;

@Controller("busquedaCotacaoController")
@Scope("prototype")
public class BusquedaCotacaoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<Cotacao> plwCotacao;

	private MyListbox cotacaolList;
	private Paging pagingCtrlCotacao;

	@Autowired
	private MonedaService monedaService;

	private Combobox cmbMoneda;
	private List<Moneda> lsMoneda;
	private org.zkoss.zul.Checkbox chcExibirInativos;

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		lsMoneda = monedaService.obterTodosExcetoReais();

		super.doAfterCompose(comp);

		cotacaolList.setItemRenderer(new RenderCotacao());
		cotacaolList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Cotacao rc = (Cotacao) cotacaolList.getSelected();
				verCotacao(rc);
			}
		});

		refreshListaPesquisa(Boolean.FALSE);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verCotacao(Cotacao c) {
		if (c == null) {
			return;
		}

		Map args = new HashMap();
		args.put("cotacao", c);
		args.put("cotacaoList", cotacaolList);
		args.put("busquedaCotacaoController", this);

		openWindow("/gui/tarifas/editarCotacao.zul",
				Labels.getLabel("editarCotacaoController.window.title"), args, MODAL);
	}

	public void refreshListaPesquisa(Boolean exibirMensagemPesquisaVazia) {
		HibernateSearchObject<Cotacao> excepcionRedondoBusqueda = new HibernateSearchObject<Cotacao>(Cotacao.class, pagingCtrlCotacao.getPageSize());

		if (!chcExibirInativos.isChecked()) {
			excepcionRedondoBusqueda.addFilterEqual("activo", Boolean.TRUE);
		}

		excepcionRedondoBusqueda.addSort("fecmodif", true);

		Moneda moneda = (Moneda) (cmbMoneda.getSelectedItem() != null ? cmbMoneda.getSelectedItem().getValue() : null);
		if (moneda != null) {
			excepcionRedondoBusqueda.addFilterEqual("moneda", moneda);
		}

		plwCotacao.init(excepcionRedondoBusqueda, cotacaolList, pagingCtrlCotacao);

		if (cotacaolList.getData().length == 0 && exibirMensagemPesquisaVazia) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("editarCotacaoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshListaPesquisa(Boolean.TRUE);
	}

	public void onClick$btnRefresh(Event ev) {
		refreshListaPesquisa(Boolean.TRUE);
	}

	public void onClick$btnNovo(Event ev) {
		verCotacao(new Cotacao());
	}

	public PagedListWrapper<Cotacao> getPlwCotacao() {
		return plwCotacao;
	}

	public void setPlwCotacao(PagedListWrapper<Cotacao> plwCotacao) {
		this.plwCotacao = plwCotacao;
	}

	public MyListbox getCotacaolList() {
		return cotacaolList;
	}

	public void setCotacaolList(MyListbox cotacaolList) {
		this.cotacaolList = cotacaolList;
	}

	public Paging getPagingCtrlCotacao() {
		return pagingCtrlCotacao;
	}

	public void setPagingCtrlCotacao(Paging pagingCtrlCotacao) {
		this.pagingCtrlCotacao = pagingCtrlCotacao;
	}

	public MonedaService getMonedaService() {
		return monedaService;
	}

	public void setMonedaService(MonedaService monedaService) {
		this.monedaService = monedaService;
	}

	public Combobox getCmbMoneda() {
		return cmbMoneda;
	}

	public void setCmbMoneda(Combobox cmbMoneda) {
		this.cmbMoneda = cmbMoneda;
	}

	public List<Moneda> getLsMoneda() {
		return lsMoneda;
	}

	public void setLsMoneda(List<Moneda> lsMoneda) {
		this.lsMoneda = lsMoneda;
	}

	public org.zkoss.zul.Checkbox getChcExibirInativos() {
		return chcExibirInativos;
	}

	public void setChcExibirInativos(org.zkoss.zul.Checkbox chcExibirInativos) {
		this.chcExibirInativos = chcExibirInativos;
	}
}
